@@ -2,19 +2,18 @@
  FILE: icalspanlist.c
  CREATOR: ebusboom 23 aug 2000
 
- (C) COPYRIGHT 2000, Eric Busboom <eric@softwarestudio.org>
-     http://www.softwarestudio.org
+ (C) COPYRIGHT 2000, Eric Busboom <eric@civicknowledge.com>
 
- This program is free software; you can redistribute it and/or modify
+ This library is free software; you can redistribute it and/or modify
  it under the terms of either:
 
     The LGPL as published by the Free Software Foundation, version
-    2.1, available at: http://www.gnu.org/licenses/lgpl-2.1.html
+    2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.html
 
  Or:
 
-    The Mozilla Public License Version 1.0. You may obtain a copy of
-    the License at http://www.mozilla.org/MPL/
+    The Mozilla Public License Version 2.0. You may obtain a copy of
+    the License at https://www.mozilla.org/MPL/
  ======================================================================*/
 
 #ifdef HAVE_CONFIG_H
@@ -83,23 +82,11 @@ static void icalspanlist_new_callback(icalcomponent *comp, struct icaltime_span
         return;
     }
 
-  /** copy span data into allocated memory.. **/
+  /* copy span data into allocated memory.. **/
     *s = *span;
     pvl_insert_ordered(sl->spans, compare_span, (void *)s);
 }
 
-/** @brief Make a free list from a set of VEVENT components.
- *
- *  @param set    A valid icalset containing VEVENTS
- *  @param start  The free list starts at this date/time
- *  @param end    The free list ends at this date/time
- *
- *  @return        A spanlist corresponding to the VEVENTS
- *
- * Given a set of components, a start time and an end time
- * return a spanlist that contains the free/busy times.
- */
-
 icalspanlist *icalspanlist_new(icalset *set, struct icaltimetype start, struct icaltimetype end)
 {
     struct icaltime_span range;
@@ -121,7 +108,7 @@ icalspanlist *icalspanlist_new(icalset *set, struct icaltimetype start, struct i
     range.start = icaltime_as_timet(start);
     range.end = icaltime_as_timet(end);
 
-    /* Get a list of spans of busy time from the events in the set
+    /* Gets a list of spans of busy time from the events in the set
        and order the spans based on the start time */
 
     for (c = icalset_get_first_component(set);
@@ -152,6 +139,9 @@ icalspanlist *icalspanlist_new(icalset *set, struct icaltimetype start, struct i
     for (itr = pvl_head(sl->spans); itr != 0; itr = pvl_next(itr)) {
         struct icaltime_span *s = (struct icaltime_span *)pvl_data(itr);
 
+        if (!s)
+            continue;
+
         if ((freetime = (struct icaltime_span *)malloc(sizeof(struct icaltime_span))) == 0) {
             icalerror_set_errno(ICAL_NEWFAILED_ERROR);
             icalspanlist_free(sl);
@@ -199,12 +189,6 @@ icalspanlist *icalspanlist_new(icalset *set, struct icaltimetype start, struct i
     return sl;
 }
 
-/** @brief Destructor.
- *  @param s A valid icalspanlist
- *
- *  Free memory associated with the spanlist
- */
-
 void icalspanlist_free(icalspanlist *s)
 {
     struct icaltime_span *span;
@@ -223,10 +207,6 @@ void icalspanlist_free(icalspanlist *s)
     free(s);
 }
 
-/** @brief (Debug) print out spanlist to stdout.
- *  @param sl A valid icalspanlist.
- */
-
 void icalspanlist_dump(icalspanlist *sl)
 {
     int i = 0;
@@ -234,24 +214,15 @@ void icalspanlist_dump(icalspanlist *sl)
 
     for (itr = pvl_head(sl->spans); itr != 0; itr = pvl_next(itr)) {
         struct icaltime_span *s = (struct icaltime_span *)pvl_data(itr);
-
-        printf("#%02d %d start: %s", ++i, s->is_busy, ctime(&s->start));
-        printf("      end  : %s", ctime(&s->end));
+        if (s) {
+            printf("#%02d %d start: %s", ++i, s->is_busy, ctime(&s->start));
+            printf("      end  : %s", ctime(&s->end));
+        }
     }
 }
 
-icalcomponent *icalspanlist_make_free_list(icalspanlist *sl);
 icalcomponent *icalspanlist_make_busy_list(icalspanlist *sl);
 
-/** @brief Find next free time span in a spanlist.
- *
- *  @param  sl     The spanlist to search.
- *  @param  t      The time to start looking.
- *
- *  Given a spanlist and a time, find the next period of time
- *  that is free
- */
-
 struct icalperiodtype icalspanlist_next_free_time(icalspanlist *sl, struct icaltimetype t)
 {
     pvl_elem itr;
@@ -262,6 +233,7 @@ struct icalperiodtype icalspanlist_next_free_time(icalspanlist *sl, struct icalt
 
     period.start = icaltime_null_time();
     period.end = icaltime_null_time();
+    period.duration = icaldurationtype_null_duration();
 
     itr = pvl_head(sl->spans);
     s = (struct icaltime_span *)pvl_data(itr);
@@ -279,9 +251,9 @@ struct icalperiodtype icalspanlist_next_free_time(icalspanlist *sl, struct icalt
         period.start = t;
 
         if (s->is_busy == 1) {
-            period.end = icaltime_from_timet(s->start, 0);
+            period.end = icaltime_from_timet_with_zone(s->start, 0, NULL);
         } else {
-            period.end = icaltime_from_timet(s->end, 0);
+            period.end = icaltime_from_timet_with_zone(s->end, 0, NULL);
         }
 
         return period;
@@ -292,15 +264,18 @@ struct icalperiodtype icalspanlist_next_free_time(icalspanlist *sl, struct icalt
     for (itr = pvl_head(sl->spans); itr != 0; itr = pvl_next(itr)) {
         s = (struct icaltime_span *)pvl_data(itr);
 
+        if (!s)
+            continue;
+
         if (s->is_busy == 0 && s->start >= rangett && (rangett < s->end || s->end == s->start)) {
 
             if (rangett < s->start) {
-                period.start = icaltime_from_timet(s->start, 0);
+                period.start = icaltime_from_timet_with_zone(s->start, 0, NULL);
             } else {
-                period.start = icaltime_from_timet(rangett, 0);
+                period.start = icaltime_from_timet_with_zone(rangett, 0, NULL);
             }
 
-            period.end = icaltime_from_timet(s->end, 0);
+            period.end = icaltime_from_timet_with_zone(s->end, 0, NULL);
 
             return period;
         }
@@ -312,28 +287,6 @@ struct icalperiodtype icalspanlist_next_free_time(icalspanlist *sl, struct icalt
     return period;
 }
 
-struct icalperiodtype icalspanlist_next_busy_time(icalspanlist *sl, struct icaltimetype t);
-
-/** @brief Returns an hour-by-hour array of free/busy times over a
- *         given period.
- *
- *  @param sl        A valid icalspanlist
- *  @param delta_t   The time slice to divide by, in seconds.  Default 3600.
- *
- *  @return A pointer to an array of integers containing the number of
- *       busy events in each delta_t time period.  The final entry
- *       contains the value -1.
- *
- *  This calculation is somewhat tricky.  This is due to the fact that
- *  the time range contains the start time, but does not contain the
- *  end time.  To perform a proper calculation we subtract one second
- *  off the end times to get a true containing time.
- *
- *  Also note that if you supplying a spanlist that does not start or
- *  end on a time boundary divisible by delta_t you may get results
- *  that are not quite what you expect.
- */
-
 int *icalspanlist_as_freebusy_matrix(icalspanlist *sl, int delta_t)
 {
     pvl_elem itr;
@@ -347,11 +300,11 @@ int *icalspanlist_as_freebusy_matrix(icalspanlist *sl, int delta_t)
     if (!delta_t)
         delta_t = 3600;
 
-  /** calculate the start and end time as time_t **/
+  /* calculate the start and end time as time_t **/
     sl_start = icaltime_as_timet_with_zone(sl->start, icaltimezone_get_utc_timezone());
     sl_end = icaltime_as_timet_with_zone(sl->end, icaltimezone_get_utc_timezone());
 
-  /** insure that the time period falls on a time boundary divisable
+  /* insure that the time period falls on a time boundary divisible
       by delta_t */
 
     sl_start /= delta_t;
@@ -360,18 +313,18 @@ int *icalspanlist_as_freebusy_matrix(icalspanlist *sl, int delta_t)
     sl_end /= delta_t;
     sl_end *= delta_t;
 
-  /** find the duration of this spanlist **/
+  /* find the duration of this spanlist **/
     spanduration_secs = sl_end - sl_start;
 
-  /** malloc our matrix, add one extra slot for a final -1 **/
+  /* malloc our matrix, add one extra slot for a final -1 **/
     matrix_slots = spanduration_secs / delta_t + 1;
 
-    matrix = (int *)malloc(sizeof(int) * matrix_slots);
+    matrix = (int *)malloc((size_t)(sizeof(int) * matrix_slots));
     if (matrix == NULL) {
         icalerror_set_errno(ICAL_NEWFAILED_ERROR);
         return NULL;
     }
-    memset(matrix, 0, sizeof(int) * matrix_slots);
+    memset(matrix, 0, (size_t)(sizeof(int) * matrix_slots));
     matrix[matrix_slots - 1] = -1;
 
     /* loop through each span and mark the slots in the array */
@@ -379,7 +332,7 @@ int *icalspanlist_as_freebusy_matrix(icalspanlist *sl, int delta_t)
     for (itr = pvl_head(sl->spans); itr != 0; itr = pvl_next(itr)) {
         struct icaltime_span *s = (struct icaltime_span *)pvl_data(itr);
 
-        if (s->is_busy == 1) {
+        if (s && s->is_busy == 1) {
             time_t offset_start = s->start / delta_t - sl_start / delta_t;
             time_t offset_end = (s->end - 1) / delta_t - sl_start / delta_t + 1;
             time_t i;
@@ -395,27 +348,12 @@ int *icalspanlist_as_freebusy_matrix(icalspanlist *sl, int delta_t)
     return matrix;
 }
 
-/** @brief Return a VFREEBUSY component for the corresponding spanlist
- *
- *   @param sl         A valid icalspanlist, from icalspanlist_new()
- *   @param organizer  The organizer specified as MAILTO:user@domain
- *   @param attendee   The attendee specified as MAILTO:user@domain
- *
- *   @return            A valid icalcomponent or NULL.
- *
- * This function returns a VFREEBUSY component for the given spanlist.
- * The start time is mapped to DTSTART, the end time to DTEND.
- * Each busy span is represented as a separate FREEBUSY entry.
- * An attendee parameter is required, and organizer parameter is
- * optional.
- */
-
 icalcomponent *icalspanlist_as_vfreebusy(icalspanlist *sl,
                                          const char *organizer, const char *attendee)
 {
     icalcomponent *comp;
     icalproperty *p;
-    struct icaltimetype atime = icaltime_from_timet(time(0), 0);
+    struct icaltimetype atime = icaltime_from_timet_with_zone(time(0), 0, NULL);
     pvl_elem itr;
     icaltimezone *utc_zone;
     icalparameter *param;
@@ -444,7 +382,7 @@ icalcomponent *icalspanlist_as_vfreebusy(icalspanlist *sl,
         struct icalperiodtype period;
         struct icaltime_span *s = (struct icaltime_span *)pvl_data(itr);
 
-        if (s->is_busy == 1) {
+        if (s && s->is_busy == 1) {
 
             period.start = icaltime_from_timet_with_zone(s->start, 0, utc_zone);
             period.end = icaltime_from_timet_with_zone(s->end, 0, utc_zone);
@@ -461,15 +399,6 @@ icalcomponent *icalspanlist_as_vfreebusy(icalspanlist *sl,
     return comp;
 }
 
-/** @brief Return a spanlist corresponding to the VFREEBUSY portion of
- *         an icalcomponent.
- *
- *   @param   c        A valid icalcomponent.
- *
- *   @return           A valid icalspanlist or NULL if no VFREEBUSY section.
- *
- */
-
 icalspanlist *icalspanlist_from_vfreebusy(icalcomponent *comp)
 {
     icalcomponent *inner;
